const Generator = require('yeoman-generator');
const { execSync } = require('child_process');
const { EOL } = require('os');

const capitalise = (word) => {
  return word[0].toUpperCase() + word.slice(1);
};

module.exports = class extends Generator {
  prompting() {
    const done = this.async();

    const plural = {
      type: 'input',
      name: 'plural',
      message: 'Plural resource name?',
    };

    const singular = {
      type: 'input',
      name: 'singular',
      message: 'Singular resource name?',
    };

    return this.prompt([
      plural,
      singular,
    ]).then((answers) => {
      this.answers = {
        plural: answers.plural.toLowerCase(),
        singular: answers.singular.toLowerCase(),
      };

      done();
    });
  }

  writing() {
    const {
      plural,
      singular,
    } = this.answers;

    const Plural = capitalise(plural);
    const Singular = capitalise(singular);
    const SINGULAR = singular.toUpperCase();

    const fileTemplates = [
      'src/containers/Resource.js',
      'src/containers/ResourceList.js',
      'src/stores/ResourceStore.js',
      'src/stores/ResourceListStore.js',
      'src/components/ResourceForm.js',
    ];

    fileTemplates.forEach((filename) => {
      this.fs.copyTpl(
        this.templatePath(filename),
        this.destinationPath(filename.replace(/Resource/, Singular)),
        {
          Plural,
          Singular,
          plural,
          singular,
          SINGULAR,
        }
      );
    });

    // Update src/App.js
    const App = this.fs.read(this.destinationPath('src/App.js'));

    const storeImport = `${EOL}import ${Singular}Store from './stores/${Singular}Store';`;
    const storeCreate = `${EOL}  ${singular}Store: new ${Singular}Store(),`;

    const listStoreImport = `${EOL}import ${Singular}ListStore from './stores/${Singular}ListStore';`;
    const listStoreCreate = `${EOL}  ${singular}ListStore: new ${Singular}ListStore(),`;

    let importStoresDone = false;
    let createStoresDone = false;

    const newApp = App.toString().split(EOL).map((line) => {
      if (line.match(/\/stores\//) && !importStoresDone) {
        line += storeImport;
        line += listStoreImport;

        importStoresDone = true;
      }

      if (line.match(/const stores/) && !createStoresDone) {
        line += storeCreate;
        line += listStoreCreate;

        createStoresDone = true;
      }

      return line;
    }).join(EOL);

    this.fs.write(this.destinationPath('src/App.js'), newApp);

    // Update src/Navigation.js
    const Navigation = this.fs.read(this.destinationPath('src/Navigation.js'));

    const item = `${EOL}  { name: '${Plural}', url: '/${plural}', icon: 'fas fa-list' },`;

    let itemDone = false;

    const newNavigation = Navigation.toString().split(EOL).map((line) => {
      if (line.match(/const items/) && !itemDone) {
        line += item;
        itemDone = true;
      }

      return line;
    }).join(EOL);

    this.fs.write(this.destinationPath('src/Navigation.js'), newNavigation);

    // Update src/Router.js
    const Router = this.fs.read(this.destinationPath('src/Router.js'));


    const newResourceRoute = `${EOL}  { path: '/${plural}/new', exact: true, name: 'New ${Singular}', component: ${Singular} },`;
    const resourceImport = `${EOL}import ${Singular} from './containers/${Singular}';`;
    const resourceRoute = `${EOL}  { path: '/${plural}/:uuid', exact: true, name: '${Singular}', component: ${Singular} },`;

    const resourceListImport = `${EOL}import ${Singular}List from './containers/${Singular}List';`;
    const resourceListRoute = `${EOL}  { path: '/${plural}', exact: true, name: '${Plural}', component: ${Singular}List },`;

    let resourceImportsDone = false;
    let resourceRoutesDone = false;

    const newRouter = Router.toString().split(EOL).map((line) => {
      if (line.match(/\/containers\//) && !resourceImportsDone) {
        line += resourceImport;
        line += resourceListImport;

        resourceImportsDone = true;
      }

      if (line.match(/const routes/) && !resourceRoutesDone) {
        line += newResourceRoute;
        line += resourceRoute;
        line += resourceListRoute;

        resourceRoutesDone = true;
      }

      return line;
    }).join(EOL);

    this.fs.write(this.destinationPath('src/Router.js'), newRouter);
  }
};

